@@ -394,24 +394,32 @@ e_contact_entry_set_sources (EContactEntry *entry, ESourceGroup *sources)
   GSList *s;
 
   g_return_if_fail (E_IS_CONTACT_ENTRY (entry));
-  
-  /* Unref the old sources */
-  if (sources == NULL) {
-    GList *l = NULL;
-    gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
-    for (l = entry->lookup_entries; l != NULL; l = g_list_next (l)) {
-      lookup_entry_free (l->data);
-    }
+
+  /*
+   * TODO: define policy. Does this take a reference to the source, so calling
+   * code should unref; or does this own the copy passed to it?
+   */
+
+  /* Release the old sources */  
+  if (entry->lookup_entries) {
+    g_list_foreach (entry->lookup_entries, (GFunc)lookup_entry_free, NULL);
     g_list_free (entry->lookup_entries);
+  }
+  if (entry->sources) {
+    g_object_unref (entry->sources);
+  }
+
+  /* If we have no new sources, disable and return here */
+  if (sources == NULL) {
+    gtk_widget_set_sensitive (GTK_WIDGET (entry), FALSE);
+    entry->sources = NULL;
+    entry->lookup_entries = NULL;
     return;
   }
 
   entry->sources = sources;
 
-  g_list_foreach (entry->lookup_entries, (GFunc)lookup_entry_free, NULL);
-  g_list_free (entry->lookup_entries);
   entry->lookup_entries = NULL;
-
   for (s = e_source_group_peek_sources (sources); s != NULL; s = s->next) {
     EntryLookup *lookup;
     lookup = g_new0 (EntryLookup, 1);
@@ -421,8 +429,9 @@ e_contact_entry_set_sources (EContactEntry *entry, ESourceGroup *sources)
       /* TODO handle this better, fire the error signal I guess */
       g_warning (error->message);
       g_error_free (error);
+      g_free (lookup);
     } else {
-      entry->lookup_entries = g_list_prepend (entry->lookup_entries, lookup);
+      entry->lookup_entries = g_list_append (entry->lookup_entries, lookup);
       e_book_async_open(lookup->book, TRUE, (EBookCallback)book_opened_cb, lookup);
     }
   }
@@ -629,12 +638,19 @@ e_contact_entry_new (void)
 static void
 lookup_entry_free (EntryLookup *lookup)
 {
+  /* We didn't take a reference for -> entry so don't unref it here. Yet. */
+  g_return_if_fail (lookup != NULL);
   /* TODO: unref entry if we ref it anywhere */
   if (lookup->bookview) {
-    g_warning("ebookview still around");
+    g_warning("EBookView still around");
     g_object_unref (lookup->bookview);
   }
-  g_object_unref (lookup->book);
+  if (lookup->book) {
+    g_object_unref (lookup->book);
+  } else {
+    g_warning ("EntryLookup object with no book member");
+  }
+  g_free (lookup);
 }
 
 /**
